/* Modulo: Color.c
 *
 * Curso: Computao Grfica - 2000
 * Ultima alterao: 27/02/2000
 */
#include <stdio.h>
#include <math.h>
#include "color.h"

static unsigned char Clamp(double val) {
  if ( val < 0 ) 
    return 0;
  else if ( val>255 ) 
    return 255;
  else
    return ((unsigned char)val);
}

Color  corCreate(unsigned char red, unsigned char green, unsigned char blue) {
  Color c;
  c.r = Clamp(red);
  c.g = Clamp(green);
  c.b = Clamp(blue);
  return c;
}

Color  corRead(FILE *fp) {
  Color c;
  unsigned char red,green,blue;
  if (fscanf(fp," %ld %ld %ld ",&red, &green, &blue)!=3) 
    printf("***Erro em corRead (%ld,%ld,%ld) \n",red,green,blue);
  c.r = Clamp(red);
  c.g = Clamp(green);
  c.b = Clamp(blue);
  return c;
}
 
Color  corCopy(Color c1) {
  Color c;
  c.r = c1.r;
  c.g = c1.g;
  c.b = c1.b;
  return c;
}


Color  corAdd(Color c1, Color c2) {
  Color c;
  c.r = Clamp(c1.r+c2.r);
  c.g = Clamp(c1.g+c2.g);
  c.b = Clamp(c1.b+c2.b);
  return c;
}

double  corSqrMinus(Color c1, Color c2) {
  double sqr = (c1.r - c2.r)*(c1.r - c2.r)
           +(c1.g - c2.g)*(c1.g - c2.g)
           +(c1.b - c2.b)*(c1.b - c2.b);
  return sqr;
}

double  corMinus(Color c1, Color c2) {
  double euc = (c1.r - c2.r)*(c1.r - c2.r)
           +(c1.g - c2.g)*(c1.g - c2.g)
           +(c1.b - c2.b)*(c1.b - c2.b);
  euc = sqrt(euc);
  return euc;
}


Color  corScale(double s, Color c1) {
  Color c;
  c.r = Clamp(s*c1.r);
  c.g = Clamp(s*c1.g);
  c.b = Clamp(s*c1.b);
  return c;
}

Color  corReflect(double s, Color light, Color k) {
  Color c;
  c.r = Clamp(s*(light.r*k.r));
  c.g = Clamp(s*(light.g*k.g));
  c.b = Clamp(s*(light.b*k.b));
  return c;
}

unsigned char corRed(Color c){
  return c.r;
}

unsigned char corGreen(Color c) {
  return c.g;
}

unsigned char corBlue(Color c) {
  return c.b;
}
