/* Modulo: Implementao do TAD: Imagem
 *
 * Curso: Computao Grfica - 2000
 * Ultima alterao: 20mar2000
 */
  
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <malloc.h>
#include <math.h>
#include "image.h"
#include "color.h"
#include "hash.h"

//define a estrutura Image
struct Image_impl{
   int   h;                  /* nmero de linhas da Image */
   int   w;                  /* nmero de colunas da Image */
   unsigned char *red;       /* matriz das componentes vermelha  */
   unsigned char *green;     /* matriz das componentes vermelha  */
   unsigned char *blue;      /* matriz das componentes vermelha  */
};

typedef struct Image_impl Image;








/*
  Funes auxiliares 
*/


double   CharToReal      (unsigned char mono_cor)
{
   double mono_cor2;

   mono_cor2 = (double) ((double)mono_cor/255.0);

   return (mono_cor2);

}



unsigned char RealToChar      (double mono_cor)
{
   unsigned char mono_cor2;

   mono_cor2 = (unsigned char) (255*mono_cor);

   return (mono_cor2);
}



static unsigned char ClampChar(double x)
{
  if (x>255)
    x=255.;
  if (x<0)
    x=0.;
  return ((unsigned char) x);
}



double  SqrMinus(unsigned char r1, unsigned char g1, unsigned char b1, unsigned char r2, unsigned char g2, unsigned char b2) {
  double sqr = (r1 - r2)*(r1 - r2)
           +(g1 - g2)*(g1 - g2)
           +(b1 - b2)*(b1 - b2);
  return sqr;
}







/*
Mtodos relativos a Image
*/

Image *imgCreate(int _w, int _h){
   Image *img;
   if ( _w<=0 || _h<=0)  return NULL;
   img=(Image*) malloc(sizeof(Image));
   if (img!=NULL ){
      img->w=_w; 
      img->h=_h; 
      img->red   = (unsigned char *) malloc (_w * _h * sizeof(unsigned char) ); 
      img->green = (unsigned char *) malloc (_w * _h * sizeof(unsigned char) ); 
      img->blue  = (unsigned char *) malloc (_w * _h * sizeof(unsigned char) ); 

      if (img->red==NULL || img->green==NULL || img->blue==NULL ) { 
           if (img->red  !=NULL) free(img->red);
           if (img->green!=NULL) free(img->green);
           if (img->blue !=NULL) free(img->blue);
           free (img);
           return NULL;
      }

  }
  return img;
}


void imgDestroy(Image * img){
  if (img !=NULL) {
     if (img->red)    free (img->red);
     if (img->green)  free (img->green);
     if (img->blue)   free (img->blue);

     img->red    = NULL;
     img->green = NULL;
     img->blue    = NULL;


      free(img);
     img = NULL;
  }
}





unsigned char imgGetPixelRed(Image *img, int x, int y) {  
    return img->red[y*img->w+x];
}

unsigned char imgGetPixelGreen(Image *img, int x, int y) {  
    return img->green[y*img->w+x];
}

unsigned char imgGetPixelBlue(Image *img, int x, int y) {
    return img->blue[y*img->w+x];
}


void imgSetPixel(Image *img, int x, int y, unsigned char r, unsigned char g, unsigned char b){
  img->red[y*img->w+x] = r;
  img->green[y*img->w+x] = g;
  img->blue[y*img->w+x] = b;
  return;
}

int     imgGetWidth(Image *img) {  
    return img->w;
}

int     imgGetHeight(Image *img) {
    return img->h;
}


unsigned char *imgGetRedChannel(Image *img) {
    return img->red;
}

unsigned char *imgGetGreenChannel(Image *img) {
    return img->green;
}

unsigned char *imgGetBlueChannel(Image *img) {
    return img->blue;
}



/* 
Funes clientes das acima 
*/

Image * imgReadPPM(char *filename) {
   Image *img=NULL; /* imagem criada */
   FILE *fp;
   char line[256];
   unsigned char pixel[3];
   int x,y;
   int w,h,maxcolor;

   fp = fopen(filename, "rb");
   if (fp == NULL){
      printf("%s nao pode ser aberto\n",filename);
      return img;
   }
  fgets(line,256,fp);

  if(strcmp(line,"P6\n")) {
     return img;
  }

  while (fscanf( fp, " %d ", &w ) != 1)
     fgets(line, 256, fp);
  while (fscanf( fp, " %d ", &h ) != 1)
     fgets(line, 256, fp);
  while (fscanf( fp, " %d", &maxcolor ) != 1)
     fgets(line, 256, fp);
  
  fgetc(fp);

  img = imgCreate(w,h);
  if (img!=NULL) {
     /* le invertendo as linhas */
     for ( y=imgGetHeight(img)-1; y>=0; y-- ) {
        for ( x=0; x<imgGetWidth(img); x++ ) {
           fread( pixel, 3, 1, fp );
           imgSetPixel(img, x, y, pixel[0], pixel[1], pixel[2]);
        }
     }
  }
  else 
     printf("No foi possivel alocar a imagem %s \n", filename);

  fclose(fp);
  return img;
}

int imgWritePPM(Image * img, char * filename)
{
  FILE * fp;
  int  x,y;
  int  maxcolor=255;
  unsigned char red, green, blue;

  if ((fp = fopen(filename, "wb")) == NULL)
  {
    printf("\nNo foi possivel abrir o arquivo %s\n",filename);
    return 0;
  }

  /*cabealho de um arquivo .ppm */
  fprintf(fp,"P6\n%d %d\n%d\n", img->w, img->h, maxcolor);

  /* gravando invertendo o eixo y */
  for ( y=imgGetHeight(img)-1; y>=0; y-- )
    for ( x=0; x<imgGetWidth(img); x++ )
    {
      red   = imgGetPixelRed(img, x, y);
      green = imgGetPixelGreen(img, x, y);
      blue  = imgGetPixelBlue(img, x, y);
      fprintf(fp,"%c%c%c", red, green, blue);
    }

  printf("\nO %s foi gerado com sucesso.\n",filename);

  fclose(fp);
  return 1;
}




long int imgCountColors(Image *img){ 
     int x,y;
     char color[4];
     p_hash hash;
     long int order = 0, last_order = 0;
     int w = imgGetWidth(img), h = imgGetHeight(img);

     hash = hash_create();
    
     color[3] = '\0';
     for ( y=0;y<h; y++ )
         for ( x=0; x<w; x++ )
         {
               color[0] = imgGetPixelRed(img, x, y);
               color[1] = imgGetPixelGreen(img, x, y);
               color[2] = imgGetPixelBlue(img, x, y);

               hash_insert(hash, color, &order);

               if (order > last_order) last_order = order;
         }
     
    hash_kill(hash);
    return last_order+1;
}




double imgError(Image *img1, Image *img2)
{
     int w = imgGetWidth(img1);
     int h = imgGetHeight(img1);
     double error = 0.0;
     int x, y;
     unsigned char r,g,b;
     double dr,dg,db;

     if ( (w != imgGetWidth(img2)) || (h != imgGetHeight(img2)) )
      return(-1.0);


     for (y=0; y < h; y++)
         for (x=0; x < w; x++){
               dr = (double) imgGetPixelRed(img1, x, y) - 
                    (double) imgGetPixelRed(img2, x, y);
               dg = (double) imgGetPixelGreen(img1, x, y) - 
                    (double) imgGetPixelGreen(img2, x, y);
               db = (double) imgGetPixelBlue(img1, x, y)  - 
                    (double) imgGetPixelBlue(img2, x, y);

               error += sqrt ( dr*dr + db*db + db*db );

               dr=(dr>0)?dr:-dr;
               dg=(dg>0)?dg:-dg;
               db=(db>0)?db:-db;

               r=(unsigned char)dr;
               g=(unsigned char)dg;
               b=(unsigned char)db;

               imgSetPixel(img2,x,y,r,g,b);
               
          }

          error /= (double)(w*h);

     return(error);

}




