/*=========================================================================*\
* Hash table implementation
* Diego Nehab
* 21/9/1999
\*=========================================================================*/

#ifndef _HASH_H_
#define _HASH_H_

#include "hashtypes.h"

/*=========================================================================*\
* Exported function declarations
\*=========================================================================*/
/*-------------------------------------------------------------------------*\
* Creates a new hash table
* Returns
*   pointer to newly created hash table in case of success. NULL
*   othrewise
\*-------------------------------------------------------------------------*/
p_hash hash_create(void);

/*-------------------------------------------------------------------------*\
* Frees all memory occupied by hash table
* Input
*   hash: hash table
* Returns
*   1 in case of success, 0 otherwise
\*-------------------------------------------------------------------------*/
int hash_kill(p_hash hash);

/*-------------------------------------------------------------------------*\
* Inserts a new table entry
* Input
*   hash: hash table
*   ident: entry name. a null pointer is ignored
* Output
*   order: number of elements inserted in the table before the entry. 
*     a null pointer is ignored
* Returns
*   0 if error, 1 if new element, 2 if dupplicate (data is not altered
*   in this case)
* Obs
*   a copy of ident is stored in the table
\*-------------------------------------------------------------------------*/
int hash_insert(p_hash hash, const char *ident, long int *order);

/*-------------------------------------------------------------------------*\
* Iterate on table elements in insertion order
* Input
*   hash: hash table
*   callback: function to be called on each element
* Returns
*   1 in case of success, 0 otherwise
\*-------------------------------------------------------------------------*/
int hash_iterate(p_hash hash, p_hashcallback callback);

/*-------------------------------------------------------------------------*\
* Retrieve an entry from the table
* Input
*   hash: hash table
*   ident: entry name
* Output
*   order: number of elements inserted in the table before the entry. 
*     a null pointer is ignored
*   num: number of color occurences
* Returns
*   0 if error or not found, 1 if found
\*-------------------------------------------------------------------------*/
int hash_get(p_hash hash, const char *ident, long int *num,
	long int *order);

/*-------------------------------------------------------------------------*\
* Frees all memory occupied by hash table
* Input
*   hash: hash table
* Returns
*   1 in case of success, 0 otherwise
\*-------------------------------------------------------------------------*/
int hash_kill(p_hash hash);

/*-------------------------------------------------------------------------*\
* Macros to access node fields 
\*-------------------------------------------------------------------------*/
#define hash_getidentbyorder(h,o) ((const char*)(h->table[h->index[o]].ident))
#define hash_getnumbyorder(h,o) (h->table[h->index[o]].num)
#define hash_setnumbyorder(h,o,d) (h->table[h->index[o]].num = d)
#define hash_insertident(h, i) hash_insert(h, i, NULL)

#endif /* _HASH_H_ */
