/*===========================================================================
%M Programa para correcao do trabalho 2.
%p Curso de Computacao Grafica, Marcelo Gattass.
===========================================================================*/

/*- Bibliotecas padrao usadas: --------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

/*- Inclusao das bibliotecas IUP e CD: ------------------------------------*/
#include "inc\iup.h"                /* prototypes do IUP */
#include "inc\cd.h"                 /* prototypes do CD */
#include "inc\cdiup.h"              /* driver do canvas do IUP */

#include "image.h"                  /* TAD para imagem */

/*- Contexto dos canvas 1 e 2:----------------------------------------------*/
typedef struct {
  Ihandle  *iup_dialog;             /* dialogo do IUP */
  Ihandle  *iup_canvas;             /* canvas do IUP */
  cdCanvas *cd_canvas;              /* canvas do CD */
  Image    *img;                    /* imagem */
} GContext;                         /* Graphical context  */


/*-------------------------------------------------------------------------*/
/* Funcao associada ao evento de repaint do canvas.                        */
/*-------------------------------------------------------------------------*/
int repaint_cb(Ihandle *self, double x, double y) {
     /* recupera o contexto grafico deste canvas */
     GContext *gch = (GContext *)IupGetAttribute(self,"gc");

     if (gch!= NULL && gch->img!=NULL) {
       int w,h;
       /* torna o canvas corrente e avisa o CD que havera um redesenho */
       cdActivate(gch->cd_canvas);

       /* obtem o tamanho do canvas */
        cdGetCanvasSize(&w, &h, NULL, NULL); 

      /* coloca a imagem na tela esticando para colocar a imagem toda */
       cdPutImageRectRGB(imgGetWidth(gch->img), imgGetHeight(gch->img), 
                         imgGetRedChannel(gch->img), imgGetGreenChannel(gch->img), imgGetBlueChannel(gch->img),
                          0, 0,  w, h, 0, 0 , 0,  0);
   }
  /* retorna o controle para o gerenciador de eventos (main loop) */
  return IUP_DEFAULT;

}


/*-------------------------------------------------------------*/
/* Funcao que carrega uma imagem (associada aos botoes "Load") */
/*-------------------------------------------------------------*/
int load_cb(Ihandle *self) {
    GContext *gch = (GContext *)IupGetAttribute(self,"gc"); /* contexto */
    Ihandle *filedlg = IupFileDlg();   /* tipo do IUP para dialogo de arquivos */
   char *filename, buffer[80];

   /* define os atributos do dialogo de arquivos */
    IupSetAttribute(filedlg, IUP_DIALOGTYPE, IUP_OPEN); 
    IupSetAttribute(filedlg, IUP_TITLE, "Selecione o arquivo "); 
    IupSetAttribute(filedlg, IUP_FILTER, "*.ppm"); 
    IupSetAttribute(filedlg, IUP_FILTERINFO, "PPM files"); 

    /* mostra o dialog na tela  no modo modal */
    IupPopup(filedlg, IUP_CENTER, IUP_CENTER);
  
    /* recupera o nome do arquivo selecionado */
    filename = IupGetAttribute(filedlg, IUP_VALUE);
    if (!filename) return 0; /* operao cancelada */

     /* le a imagem do arquivo selecionado */
    if( (gch->img = imgReadPPM(filename))==NULL) 
     return 0;

   /* prepara um string com as dimenses da imagem, por exemplo "320x200" */ 
    sprintf(buffer,"%dx%d",imgGetWidth(gch->img),imgGetHeight(gch->img));
    /* redefine o tamano do canvas */
    IupSetAttribute(gch->iup_canvas,IUP_RASTERSIZE,buffer);
    /* fora o IUP a recalcular o tamanho dos objetos do dialogo */
    IupSetAttribute(gch->iup_dialog,IUP_SIZE,NULL);

   /* redessenha o canvas */
    repaint_cb(gch->iup_canvas, 0.0, 0.0);

    /* retorna o controle para o gerenciador de eventos (main loop) */
   return IUP_DEFAULT;
}


/*-------------------------------------------------------------------------*/
/* Funcao que calcula o erro entre as imagens (associada ao botao "Error") */
/*-------------------------------------------------------------------------*/
int error_cb(Ihandle *self) {
   GContext *gch1  = (GContext *)IupGetAttribute(self,"gc1"); /* contexto do canvas 1 */
   GContext *gch2  = (GContext *)IupGetAttribute(self,"gc2"); /* contexto do canvas 2 */

    long int colors1 = imgCountColors(gch1->img);   /* conta as cores da imagem 1 */
    long int colors2 = imgCountColors(gch2->img);   /* conta as cores da imagem 2 */
   
    /* calcula o erro medio por pixel e refaz a imagem 2 com a diferenca entre 1 e 2 */
    double    error = imgError(gch1->img, gch2->img);
    
    /* exibe o dialogo de resultados */
    char buffer[80];
   sprintf(buffer,"Colors in 1 = %ld\nColors in 2 = %ld\nError = %.0lf ", colors1, colors2, error);
    IupMessage ("Measures", buffer);
    
    /* redesenha o canvas */
    repaint_cb(gch2->iup_canvas,0.0,0.0);
   
    /* retorna o controle para o gerenciador de eventos (main loop) */
    return IUP_DEFAULT;
}

/*------------------------------------------------------------------*/
/* Funcao que forca a saida do programa (associada ao botao "Exit") */
/*------------------------------------------------------------------*/
int exit_cb(Ihandle *self) {
   GContext *gch1  = (GContext *)IupGetAttribute(self,"gc1"); /* contexto 1 */
   GContext *gch2  = (GContext *)IupGetAttribute(self,"gc2"); /* contexto 2 */

    /* desaloca as imagens */
    imgDestroy(gch1->img); 
   imgDestroy(gch2->img);

    /* retorna o controle para o gerenciador de eventos pedindo que ele encerre */
    return IUP_CLOSE;
}

/*-------------------------------------------------------------------------*/
/* Incializa o programa.                                                   */
/*-------------------------------------------------------------------------*/
int init(void) {
  static GContext gc1,gc2;   /* contextos graficos associados aos canvas 1 e 2 */

  /* define os elementos do dialogo e seu arranjo */
  Ihandle *load1   = IupButton ("Load", "load_cb");
  Ihandle *exit    = IupButton ("Exit", "exit_cb");
  Ihandle *load2   = IupButton ("Load", "load_cb");
  Ihandle *error   = IupButton ("Error", "error_cb");

  Ihandle *canvas1 = IupCanvas("repaint_cb"); 
  Ihandle *canvas2 = IupCanvas("repaint_cb"); 

  Ihandle *cx1     = IupHbox(load1,exit,NULL);
  Ihandle *cx2     = IupHbox(load2,error,NULL);

  Ihandle *dialog  = IupDialog( IupHbox( 
                                        IupVbox(canvas1,cx1,NULL),
                                        IupVbox(canvas2,cx2,NULL), 
                                        NULL ) 
                               );

  /* armazena em cada elemento o contexto dos canvas associados */
  IupSetAttribute(canvas1,"gc",(char *) (&gc1) );
  IupSetAttribute(canvas2,"gc",(char *) (&gc2) );

  IupSetAttribute(load1,"gc",(char *) (&gc1) );
  IupSetAttribute(load2,"gc",(char *) (&gc2) );
  
  IupSetAttribute(error,"gc1",(char *) (&gc1) );
  IupSetAttribute(error,"gc2",(char *) (&gc2) );
  
  IupSetAttribute(exit,"gc1",(char *) (&gc1) );
  IupSetAttribute(exit,"gc2",(char *) (&gc2) );

  
  /* constroi e coloca o dialogo principal na tela */
  IupShowXY(dialog, IUP_LEFT, IUP_TOP);
  

  /* cria o canvas do CD associado ao canvas do IUP */
  gc1.cd_canvas = cdCreateCanvas(CD_IUP, canvas1 );
  gc2.cd_canvas = cdCreateCanvas(CD_IUP, canvas2 );
  
  gc1.iup_canvas=canvas1;
  gc2.iup_canvas=canvas2;
  
  gc1.iup_dialog=dialog;
  gc2.iup_dialog=dialog;

  /* desenha pela primeira vez */
  repaint_cb(canvas1,0.0,0.0);
  repaint_cb(canvas2,0.0,0.0);

  /* associa o evento de repaint a funccao repaint_cb */
  IupSetFunction("repaint_cb", (Icallback) repaint_cb);
  IupSetFunction("load_cb", (Icallback) load_cb);
  IupSetFunction("exit_cb", (Icallback) exit_cb);
  IupSetFunction("error_cb", (Icallback) error_cb);

  return 1;

}

/*-------------------------------------------------------------------------*/
/* Rotina principal.                                                       */
/*-------------------------------------------------------------------------*/
void main(void) {
    IupOpen();
    if ( init() )
      IupMainLoop();
    IupClose();
}