/** MODIFICADO POR JULIANA **/

/* Modulo: Algebra.c
 *
 * Curso: Computao Grfica - 2000
 * Ultima alterao: 26/02/2000
 */

#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include "algebra.h"

#define ZERO(X) (fabs(X)<1.0e-15)


Vector3D v3dCreate(double xl, double yl, double zl) {
  Vector3D v;
  v.x = xl;
  v.y = yl;
  v.z = zl;
  return v;
}


Vector3D v3dCopy(Vector3D v1) {
  Vector3D v;
  v.x = v1.x;
  v.y = v1.y;
  v.z = v1.z;
  return v;
}

Vector3D v3dMinus(Vector3D v1) {
  Vector3D v;
  v.x = -v1.x;
  v.y = -v1.y;
  v.z = -v1.z;
  return v;
}

Vector3D v3dAdd(Vector3D v1, Vector3D v2) {
  Vector3D v;
  v.x = v1.x + v2.x;
  v.y = v1.y + v2.y;
  v.z = v1.z + v2.z;
  return v;
}

Vector3D v3dSub(Vector3D v1, Vector3D v2) {
  Vector3D v;
  v.x = v1.x - v2.x;
  v.y = v1.y - v2.y;
  v.z = v1.z - v2.z;
  return v;
}

Vector3D v3dVaLinComb(int n, ...) {
  Vector3D v=v3dCreate(0.0,0.0,0.0);
  va_list ap;
  va_start(ap, n); 
  while (n-->0) {
       double a = va_arg(ap, double);
       Vector3D u = va_arg(ap, Vector3D);
       v.x+=a*u.x;
       v.y+=a*u.y;
       v.z+=a*u.z;
  }
  va_end(ap);
  return v;
}

Vector3D v3dScale(double s, Vector3D v2) {
  Vector3D v;
  v.x = s * v2.x;
  v.y = s * v2.y;
  v.z = s * v2.z;
  return v;
}

double v3dDot(Vector3D u, Vector3D v) {
  return v.x*u.x + v.y*u.y + v.z*u.z;
}

Vector3D v3dProj(Vector3D u, Vector3D v) {
  double s;

  s = v3dDot(u, v) / v3dDot(v, v);
  return v3dScale(s, v);
}

Vector3D v3dLinearInterpol(Vector3D v1, Vector3D v2, double t) {
  return v3dAdd(v3dScale(1.0f - t, v1), v3dScale(t, v2));
}

double v3dNorm(Vector3D v) {
  return sqrt(v.x*v.x + v.y*v.y + v.z*v.z);
}

Vector3D v3dNormalize(Vector3D v) {
  double norm;

  norm = v3dNorm(v);
  if (ZERO(norm)) {
    printf("\n[%s:%d] v3dNormalized: NULL VECTOR\n", __FILE__, __LINE__);
    return v;
  }
  else
    return v3dScale(1.0/norm, v);
}

Vector3D v3dCross(Vector3D u, Vector3D v) {
  Vector3D n;

  n.x = u.y * v.z - u.z * v.y;
  n.y = u.z * v.x - u.x * v.z;
  n.z = u.x * v.y - u.y * v.x;
  return n;
}

int v3dWrite(Vector3D v, FILE *file)
{
	if (fprintf(file, "%g %g %g", v.x, v.y, v.z) == 3) return 1;
	else return 0;
}

Vector3D v3dRead(FILE *fp)
{
	double x=0.0, y=0.0, z=0.0;

	if (fscanf(fp, "%lf %lf %lf", &x, &y, &z) != 3) {
		printf("\n[%s:%d] can't read point (%g, %g, %g)\n",
		__FILE__, __LINE__, x, y, z);
	}

	return v3dCreate(x, y, z);
}


/*==============================================================================
*Nome: Refletir
*=============================================================================*/

Vector3D Refletir( Vector3D Raio  ,
                   Vector3D Normal ) {

/* Calucula 2 * ( Raio . Normal ) * Normal - Raio */
   Vector3D RaioRefl = v3dScale( 2. * v3dDot( Raio, Normal ), Normal ) ;
   RaioRefl = v3dSub( RaioRefl, Raio ) ;

   return RaioRefl ;

} /* Refletir */
