/** MODIFICADO POR JULIANA **/

/* Implementao do mdulo CAMERA 
 *
 * Curso: Computacao Grafica - 2000
 * ultima revisao: 1/03/2000
 */

#include <stdlib.h>
#include <math.h>
#include "camera.h"

#ifndef M_PI
#define M_PI 3.1415926535
#endif

struct Camera_impl {
  /* Definio da Camara */
	Vector3D eye;   /* olho ou centro de projeo */
	Vector3D at;    /* ponto para onde a camara aponta */
	Vector3D up;    /* direo para cima */
	double fovy;    /* angulo de abertura no plano vertical da camara (em graus)*/
	double nearp;   /* distancia ao plano de clipping nearp */
	double farp;    /* distancia ao plano de clipping farp */
	int screen_w;   /* numero de pixels (colunas) na horizontal da imagem */
	int screen_h;   /* numero de pixels (linhas) na vertical da imagem */

  /* Parametros internos da Camara */
	Vector3D xe, ye, ze; /* Eixos do sistema de coordenadas da camera */
	Vector3D Pnll;       /* Ponto no canto inferior esquerdo do plano nearp */
	Vector3D Pfll;       /* Ponto no canto inferior esquerdo do plano farp */
     Vector3D un,vn;      /* Sistema de coordenadas (u,v) do plano nearp */
	Vector3D uf,vf;      /* Sistema de coordenadas (u,v) do plano farp */
};


Camera *camCreate(Vector3D eyel, Vector3D atl, Vector3D upl,
                  double fovyl, double nearpl, double farpl, 
                  int screen_wl, int screen_hl)
{
  Camera *cam = (Camera *) malloc(sizeof(Camera));
  double sx,sy,sz;
  
  /* Atribui os parametros que definem a camara */
  cam->eye = v3dCopy(eyel);
  cam->at  = v3dCopy(atl);
  cam->up  = v3dCopy(upl);
  cam->fovy = fovyl;
  cam->nearp = nearpl;
  cam->farp = farpl;
  cam->screen_w = screen_wl;
  cam->screen_h = screen_hl;

  /* Calcula o sistema de coordenadas do olho */
  cam->ze = v3dNormalize(v3dSub(cam->eye, cam->at));
  cam->xe = v3dNormalize(v3dCross(cam->up, cam->ze));
  cam->ye = v3dCross(cam->ze, cam->xe);

  /* Calcula o ponto no canto inferior esquerdo da janela do plano nearp */
  sz = nearpl;
  sy = nearpl*tan(M_PI*fovyl / ( 2. * 180. ) );
  sx = sy*screen_wl/screen_hl;
  cam->Pnll = v3dVaLinComb(4, 1.0, cam->eye, -sz, cam->ze, -sy, cam->ye, -sx, cam->xe);

  /* Calcula os eixos (u,v) do plano nearp */
  cam->un = v3dScale(2*sx/screen_wl, cam->xe);
  cam->vn = v3dScale(2*sy/screen_hl, cam->ye); /* note que e' a mesma escala que em xe */

  /* Calcula o ponto no canto inferior esquerdo da janela do plano farp */
  sx *= farpl/nearpl;
  sy *= farpl/nearpl;
  sz *= farpl/nearpl;
  cam->Pfll = v3dVaLinComb(4, 1.0, cam->eye, -sz, cam->ze, -sy, cam->ye, -sx, cam->xe);

  /* Calcula os eixos (u,v) do plano nearp */
  cam->uf = v3dScale(2*sx/screen_wl, cam->xe);
  cam->vf = v3dScale(2*sy/screen_hl, cam->ye);

  return cam;
}


Camera *camRead(FILE *fp)
{
	Vector3D eye, at, up;
	double fovy, nearp, farp;
	int screen_w, screen_h;

	eye = v3dRead(fp);
	at = v3dRead(fp);
	up = v3dRead(fp);

	if (fscanf(fp, "%lf", &fovy) != 1) return NULL;
	if (fscanf(fp, "%lf %lf", &nearp, &farp) != 2) return NULL;
	if (fscanf(fp, "%i %i", &screen_w, &screen_h) != 2) return NULL;

#ifndef NDEBUG
	printf("\n[%s:%d] read camera:\n", __FILE__, __LINE__);
	printf("\teye = (%g, %g, %g),\n", eye.x, eye.y, eye.z);
	printf("\tat = (%g, %g, %g),\n", at.x, at.y, at.z);
	printf("\tup = (%g, %g, %g),\n", up.x, up.y, up.z);
	printf("\tfovy = %g,\n", fovy);
	printf("\tnear = %g,\n", nearp);
	printf("\tfar = %g,\n", farp);
	printf("\tscreen = %dx%d,\n", screen_w, screen_h);
#endif

	return camCreate( eye, at, up, fovy, nearp, farp, screen_w, screen_h );
}


Ray camRay(Camera *cam, double x, double y) {
  Ray ray;

  ray.P0 = cam->eye ;
  ray.D = v3dVaLinComb(4, 1.0, cam->Pnll, x, cam->un, y, cam->vn, -1., cam->eye );

  return ray;
}


int camScreenW(Camera *cam) {  /* retorna o numero de colunas da imagem da camera */
	return cam->screen_w;
}

int camScreenH(Camera *cam) {  /* retorna o numero de colunas da imagem da camera */
	return cam->screen_h;
}


void camDestroy(Camera *cam) {
	free(cam);
	return;
}


/*==============================================================================
*Nome: PegarOlhoCam
*=============================================================================*/

Vector3D PegarOlhoCam( Camera* pCam ) {
   return pCam->eye ;
} /* PegarOlhoCam */
