/** MODIFICADO POR JULIANA **/

#ifndef MATERIAL_H
#define MATERIAL_H

#include <stdio.h>
#include "image.h"
#include "color.h"

typedef struct {
	/* imagem usada como textura ou NULL se nenhuma */
	/* EH RESPONSABILIDADE DE QUEM LE O MATERIAL DESTRUIR A IMAGEM */
	Image *img;

	/* Ia*Ka */
	Color ambient;
	
	/* Kd - soh utilizado se 'img' for igual a NULL */
	Color diffuse;

	/* Ks */
	Color specular;

	/* coeficiente do coseno da iluminacao especular */
	double n;
	
	/* coeficiente de reflexao (multiplica o resultado do raio refletido) */
	double reflect;

	/* indice de refracao (lei de Snell) */
	double refract;

	/* opacidade */
	double opacity;
} Material;


/* matRead - Le um material a partir do arquivo 'fp' dado. */
Material matRead( FILE *fp );


/*==============================================================================
*Nome: PegarImg
*Descrio da Funo: funo que devolve a imagem para textura para um dado 
*                     material de objeto.
*
*Parametro recebido
*   ObjMat: material do objeto.
*
*Valor Retornado
*   a imagem para textura do objeto.
*=============================================================================*/

Image* PegarImg( Material ObjMat ) ;


/*==============================================================================
*Nome: PegarAmb
*Descrio da Funo: funo que devolve a cor ambiente para um dado material de 
*                     objeto.
*
*Parametro recebido
*   ObjMat: material do objeto.
*
*Valor Retornado
*   a cor ambiente do objeto.
*=============================================================================*/

Color PegarAmb( Material ObjMat ) ;


/*==============================================================================
*Nome: PegarDif
*Descrio da Funo: funo que devolve o coeficiente de reflexo difusa para 
*                     um dado material de objeto.
*
*Parametro recebido
*   ObjMat: material do objeto.
*
*Valor Retornado
*   o coeficiente de reflexo difusa do objeto.
*=============================================================================*/

Color PegarDif( Material ObjMat ) ;


/*==============================================================================
*Nome: PegarEsp
*Descrio da Funo: funo que devolve o coeficiente de reflexo especular 
*                     para um dado material de objeto.
*
*Parametro recebido
*   ObjMat: material do objeto.
*
*Valor Retornado
*   o coeficiente de reflexo especular do objeto.
*=============================================================================*/

Color PegarEsp( Material ObjMat ) ;


/*==============================================================================
*Nome: dPegarNCoss
*Descrio da Funo: funo que devolve o coeficiente do coseno da iluminacao 
*                     especular para um dado material de objeto.
*
*Parametro recebido
*   ObjMat: material do objeto.
*
*Valor Retornado
*   o coeficiente do coseno da iluminacao especular do objeto.
*=============================================================================*/

double dPegarNCoss( Material ObjMat ) ;


/*==============================================================================
*Nome: dPegarNRefle
*Descrio da Funo: funo que devolve o coeficiente de reflexao para um dado 
*                     material de objeto.
*
*Parametro recebido
*   ObjMat: material do objeto.
*
*Valor Retornado
*   o coeficiente de reflexo do objeto.
*=============================================================================*/

double dPegarNRefle( Material ObjMat ) ;


/*==============================================================================
*Nome: dPegarNRefra
*Descrio da Funo: funo que devolve o coeficiente de refrao para um dado 
*                     material de objeto.
*
*Parametro recebido
*   ObjMat: material do objeto.
*
*Valor Retornado
*   o coeficiente de refrao do objeto.
*=============================================================================*/

double dPegarNRefra( Material ObjMat ) ;


/*==============================================================================
*Nome: dPegarOpac
*Descrio da Funo: funo que devolve a opacidade para um dado material de 
*                     objeto.
*
*Parametro recebido
*   ObjMat: material do objeto.
*
*Valor Retornado
*   a opacidade do objeto.
*=============================================================================*/

double dPegarOpac( Material ObjMat ) ;


#endif
