/** MODIFICADO POR JULIANA **/

#ifndef OBJECTS_H
#define OBJECTS_H

#include "algebra.h"
#include "material.h"


#define D_ZERO 1.e-12

/* pre-declaracao do tipo sem dar informacoes sobre o conteudo */
struct Object_impl;
typedef struct Object_impl Object;

/* 
 * Cria um novo objeto que representa uma esfera com centro em 'center' e
 * raio 'radius'. 'mat' eh o indice do material do objeto na cena.
 */
Object * objCreateSphere( int mat, Vector3D center, double radius );

/* 
 * Cria um novo objeto que representa um triangulo cujos vertices sao os
 * pontos 'v0', 'v1' e 'v2'. 'mat' eh o indice do material do objeto na cena.
 */
Object * objCreateTriangle( int mat, Vector3D v0, Vector3D v1, Vector3D v2 );

/* 
 * Cria um novo objeto que representa uma caixa paralela aos planos principais
 * com o ponto inferior esquerdo 'll' e superior direito 'ur'.
 * 'mat' eh o indice do material do objeto na cena.
 */
Object * objCreateBox( int mat, Vector3D ll, Vector3D ur );

/*
 * Libera a memoria utilizada pelo objeto 'obj'.
 */
void objDestroy( Object *obj );

/* 
 * Retorna o parametro t que indica em que ponto o raio originado em 'eye'
 * e com direcao 'ray' intercepta o objeto 'obj'. Caso o objeto nao tenha
 * intersecao com o raio, retorna zero.
 */
double objIntercept( Object *obj, Vector3D eye, Vector3D ray );

/*
 * Retorna o vetor normal ao objeto 'obj' na posicao 'pos'.
 */
Vector3D objGetNormal( Object *obj, Vector3D pos );

/*
 * Cria um objeto a partir do arquivo 'file' dado.
 */
Object * objReadSphere( FILE *file );

/*
 * Cria um objeto a partir do arquivo 'file' dado.
 */
Object * objReadTriangle( FILE *file );

/*
 * Cria um objeto a partir do arquivo 'file' dado.
 */
Object * objReadBox( FILE *file );


/*==============================================================================
*Nome: iPegarIdxMat
*Descrio da Funo: funo que devolve o ndice do material de um dado objeto.
*
*Parametro recebido
*   pObjeto: o objeto.
*
*Valor Retornado
*   o ndice do material.
*=============================================================================*/

int iPegarIdxMat( Object* pObjeto ) ;


#endif
