/*==============================================================================
*NOME:   RENDER.C
*
*  Projeto: Ray Tracing
*
*  Verso:              V 1.00
*  Autor :   Juliana Carpes Imperial        - 9714169-4
*
*Descrio do mdulo de declarao
*     Objetivo:
*        Implementar funes para o algoritmo de Ray Tracing.
*=============================================================================*/

#include <stdlib.h>

#include "render.h"
#include "object.h"

/*==============================================================================
*Nome: RayTrace
*=============================================================================*/

Color RayTrace( Scene*   pCenario ,
                Vector3D Olho     ,
					 Vector3D Direcao  ,
					 int      iNumObjs ,
					 Color    CorFundo ,
                int      iProfund  ) {
   
	int    k        ,  /* varivel auxiliar */
          idxObj   ;  /* ndice do objeto mais prximo */
	double dMinDist ;  /* a distncia do objeto mais prximo */
   
/* testa para ver se chegou na profundidade mxima da recurso */
   if ( iProfund == 0 ) {
      return corCreate( 0., 0., 0. ) ;
   } /* if */ 

/* tenta interceptar algum objeto */
   idxObj = iInterceptar( pCenario, Olho, Direcao, iNumObjs, -1, &dMinDist ) ;

   if ( idxObj > -1 ) { /* testa se raio interceptou algum objeto */

      int      iNumLuzes ;  /* nmero total de luzes */
      double   dNCoss    ;  /* potncia do cosseno */
      double   dReflec   ;  /* ndice de reflexo do objeto */
      Object*  pObjeto   ;  /* um objeto */
      Material ObjMat    ;  /* material do objeto mais prximo */
      Vector3D Posicao   ,  /* posio onde o raio intercepta o objeto */
               Normal    ,  /* a normal do objeto interseptado */
               OlhoPos   ;  /* indica a direo da pos. com o olho */
      Color    Difusa    ,  /* o coeficiente de cor difusa do objeto */
               Especular ;  /* o coeficiente de cor especular do objeto */ 
       
   /* pega o objeto mais prximo dado o seu ndice */
      pObjeto = sceGetObject( pCenario, idxObj ) ; 
   
      k = iPegarIdxMat( pObjeto ) ; /* pega o ndice do material deste objeto */
      ObjMat = sceGetMaterial( pCenario, k ) ;   /* pega o material do objeto */   
      CorFundo = PegarAmb( ObjMat ) ;          /* pega cor ambiente do objeto */
      iNumLuzes = sceGetLightCount( pCenario ) ;      /* pega nmero de luzes */
      dNCoss = dPegarNCoss( ObjMat ) ;            /* pega potncia do cosseno */
      Difusa = PegarDif( ObjMat ) ;    /* pega coeficiente de reflexo difusa */
      Especular = PegarEsp( ObjMat ) ;          /* pega coeficiente especular */
      dReflec = dPegarNRefle( ObjMat ) ;         /* pega reflexo do material */

   /* Acha a posio de interseo do raio com o objeto */
      Posicao = v3dAdd( Olho, v3dScale( dMinDist, Direcao ) ) ;
      
   /* pega normal na interseo de raio com objeto */
      Normal = objGetNormal( pObjeto, Posicao ) ;

      Normal = v3dNormalize( Normal ) ; /* normaliza a normal */

   /* acha vetor entre o olho da cmera e a posio de interseo do raio com
      o objeto */ 
      OlhoPos = v3dMinus( Direcao ) ;
      OlhoPos = v3dNormalize( OlhoPos ) ; /* normaliza OlhoPos */

   /* repete para todas as luzs */
      for ( k = 0 ; k < iNumLuzes ; k++ ) {
      
         Light    Luz         ; /* uma determinada luz do cenrio */ 
         Vector3D LuzPos      ; /* indica a direo da posio com a luz */
         double   dLuzPosNorm ; /* LusPos . Normal */
         Color    LuzCor      ; /* a cor da luz */
     
         Luz = sceGetLight( pCenario, k ) ; /* pega uma da luz do cenrio */      
     
      /* acha vetor entre a posio da luz e a posio de interseo do raio 
         com o objeto */ 
         LuzPos = v3dSub( PegarPos( Luz ), Posicao ) ;
         
      /* testa se existe um objeto entre ele a luz; se tiver, haver 
         sombra */ 
         if ( iInterceptar( pCenario, Posicao, LuzPos, iNumObjs, idxObj, 
                            &dMinDist ) != -1 ) {
            continue ;
         } /* if */

         LuzCor = PegarCor( Luz ) ; /* pega a cor da Luz */ 
         LuzPos = v3dNormalize( LuzPos ) ; /* Normaliza LuzPos */  
         dLuzPosNorm = v3dDot( LuzPos, Normal ) ; /* calcula LusPos . Normal */
         
         if ( dLuzPosNorm < 0 ) {
		    continue ;
		 } /* if */

      /* calcula a cor do pixel como C = Camb + S( Cesp + CDif ) */
         CorFundo = corAdd( CorFundo, AcharCompDif( dLuzPosNorm, Difusa, 
                                                    LuzCor ) ) ;
         CorFundo = corAdd( CorFundo, AcharCompEsp( Normal, OlhoPos, LuzPos, 
                            dLuzPosNorm, dNCoss, LuzCor, Especular ) ) ;
      } /* for */
 
      if ( dReflec != 0. ) {
         Color Cor ;
          
      /* reflete o raio para fazer reflexo */
         OlhoPos = Refletir( OlhoPos, Normal ) ;

      /* faz a recurso para fazer reflexo */
         Cor = RayTrace( pCenario, Posicao, OlhoPos, iNumObjs, CorFundo, 
                         iProfund - 1 ) ;

      /* multiplica o resultado pelo coeficiente de reflexo */
         Cor = corScale( dReflec, Cor ) ;

      /* soma o resultado com a cor at ento obtida */
         return corAdd( CorFundo, Cor ) ;
      
      } /* if */ 

   } /* if */
 
   return CorFundo ; 
     
} /* RayTrace */


/*==============================================================================
*Nome: Render
*=============================================================================*/

Image* Render( Scene* pCenario ) { 
   
   Camera*  pCam     ;  /* camra */
   Color    CorFundo ,  /* cor de fundo */
            CorPixel ;  /* Cor de um pixel */
   Image*   pImagem  ;  /* imagem */
   Vector3D Olho     ;  /* olho da cmera */
   Ray      Raio     ;  /* raio correspondente a um pixel */
	int      iNumObjs ,  /* nmero de objetos */
            iAltura  ,  /* linhas da imagem */
		      iLargura ,  /* colunas da imagem /*/
	         x        ,  /* contador para largura */
	         y        ;  /* contador para altura */

/* pega cmera do cenrio e verifica se foi possvel cri-la */
   pCam = PegarCamera( pCenario ) ;
   if ( pCam == NULL ) {
      printf( SZ_ERR_CAM ) ;
      exit( 1 ) ;   
   } /* if */

   CorFundo = PegarCorFundo( pCenario ) ; /* pega cor de fundo do cenrio */
   Olho = PegarOlhoCam( pCam ) ; /* pega olho da cmera */

/* pega o numero de linhas da imagem da cmera */
   iAltura = camScreenH( pCam ) ;

/* pega o numero de colunas da imagem da cmera */
   iLargura = camScreenW( pCam ) ;

   iNumObjs = sceGetObjectCount( pCenario ) ;

/* tenta criar uma imagem em formato PPM e testa se foi possvel cri-la */
   pImagem = imgCreate( iLargura, iAltura ) ;
   if ( pImagem == NULL ) {
      printf( SZ_ERR_IMG ) ;
      exit( 1 ) ;
   } /* if */
   
   for ( y = 0 ; y < iAltura ; y++ ) {
      for ( x = 0 ; x < iLargura ; x++ ) {

         unsigned char cVermelho , /* componente vermelha do pixel */
                       cVerde    , /* componente verde do pixel */
                       cAzul     ; /* componnte azul do pixel */
          
         Raio = camRay( pCam, x, y ) ; /* pega raio */

      /* pega a cor do pixel usando o Ray Trace */
         CorPixel = RayTrace( pCenario, Olho, Raio.D, iNumObjs, CorFundo, 3 ) ;
			
      /* pega as componentes do pixel e transorma em valores de 0 a 255 */
         cVermelho = ( unsigned char )( 255 * dPegarVerm( CorPixel ) ) ;
         cVerde = ( unsigned char )( 255 * dPegarVerde( CorPixel ) ) ;
         cAzul = ( unsigned char )( 255 * dPegarAzul( CorPixel ) ) ;

      /* coloca a cor do pixel na imagem resultante */
			imgSetPixel( pImagem, x, y, cVermelho, cVerde, cAzul ) ;
      
      } /* for */
   } /* for */            

   return pImagem ;

} /* Render */


/*==============================================================================
*Nome: AcharCompDif
*=============================================================================*/

Color AcharCompDif( double dLuzPosNorm ,
                    Color  Difusa      ,
                    Color  LuzCor       ) {

   Color Cor ; /* a cor difusa a ser retornada */
          
/* calcula Difusa * LuzCor */
   Cor = corCreate( dPegarVerm( Difusa ) * dPegarVerm( LuzCor ),
                    dPegarVerde( Difusa ) * dPegarVerde( LuzCor ),
                    dPegarAzul( Difusa ) * dPegarAzul( LuzCor ) ) ;

/* calcula ( Difusa * LuzCor ) * dLuzPosNorm */
   Cor = corScale( dLuzPosNorm, Cor ) ; 

   return Cor ;

} /* AcharCompDif */
      

/*==============================================================================
*Nome: AcharCompEsp
*=============================================================================*/

Color AcharCompEsp( Vector3D Normal      ,
                    Vector3D OlhoPos     ,
                    Vector3D LuzPos      , 
                    double   dLuzPosNorm ,
                    double   dNCoss      , 
                    Color    LuzCor      , 
                    Color    Especular    ) {
  
   Vector3D LuzPosRefl  ; /* o vetor LuzPos refletido na normal */
   Color    Cor         ;
   double   dLuzPROlhoP ; /* ( OlhoPos . LuzPosRefl )^n */

/* calcula reflexo do vetor LuzPos na Normal da Posio */
   LuzPosRefl = Refletir( LuzPos, Normal ) ;
      
/* calcula OlhoPos . LuzPosRefl */
   dLuzPROlhoP = v3dDot( LuzPosRefl, OlhoPos ) ;

/* calcula ( OlhoPos . LuzPosRefl )^dNCoss */
   dLuzPROlhoP = pow( dLuzPROlhoP, dNCoss ) ;

/* calcula Especular * LuzCor */
   Cor = corCreate( dPegarVerm( Especular ) * dPegarVerm( LuzCor ),
                    dPegarVerde( Especular ) * dPegarVerde( LuzCor ),
                    dPegarAzul( Especular ) * dPegarAzul( LuzCor ) ) ;

/* calcula ( Especular * LuzCor ) * dLuzPROlhoP e soma com a cor especualar
   da outra luz */
   Cor = corScale( dLuzPROlhoP, Cor ) ;

   return Cor ;

} /* AcharCompEsp */


/*==============================================================================
*Nome: iInterceptar
*=============================================================================*/

int iInterceptar( Scene*   pCenario  ,
                  Vector3D Olho      ,
					   Vector3D Direcao   ,
					   int      iNumObjs  , 
                  int      idxObjExc ,
                  double*  dMinDist   ) {

   int      k                 ,
            idxObj   = -1     ;  /* ndice do objeto mais prximo */
	double   dObjDist          ,  /* distncia de um dado objeto */
            dMin     = 1.e308 ;  /* distncia do objeto mais prximo */
   Object*  pObjeto           ;  /* um objeto */


   for ( k = 0 ; k < iNumObjs ; k++ ) {

      if ( k != idxObjExc ) {
	      
         pObjeto = sceGetObject( pCenario, k ) ; /* pega o k-simo objeto */

      /* v se o raio intercepta o objeto */
         dObjDist = objIntercept( pObjeto, Olho, Direcao ) ;

      /* se a distncia do objeto  maior que 1 e e' menor que a menor 
         distncia at ento, a menor distncia e o ndice de objeto mais 
         prximo passam a ser o deste objeto */
		   if ( dObjDist > 0.01
		     && dObjDist < dMin ) {
		        dMin = dObjDist ;
              idxObj = k ;
		   } /* if */
      
      } /* if */
   
   } /* for */

   *dMinDist = dMin ;
   return idxObj ;

} /* iInterceptar */


/* render.c */