/*==============================================================================
*NOME:   RENDER.H
*
*  Projeto: Ray Tracing
*
*  Verso:              V 1.00
*  Autor :   Juliana Carpes Imperial        - 9714169-4
*
*Descrio do mdulo de declarao
*     Objetivo:
*        Definir funes para o algoritmo de Ray Tracing.
*=============================================================================*/


#if !defined( render_MOD )
#define       render_MOD

#include "scene.h"


#define SZ_ERR_CAM "Impossivel criar camera!\n"
#define SZ_ERR_IMG "Impossivel criar imagem!\n"


/*==============================================================================
*Nome: RayTrace
*Descrio da Funo: funo que devolve a cor de um pixel de um cenrio dados
*                     um olho e uma direo.
*
*Parametros recebidos
*   pCenario: o cenario;
*   Olho:     o olho da cmera;
*   Direcao:  a direo do raio;
*   iNumObjs: nmero de objetos;
*   CorFundo: a cor de fundo;
*   iProfund: profundidade da recurso.
*
*Valor Retornado
*   a cor do pixel
*=============================================================================*/

Color RayTrace( Scene*   pCenario ,
                Vector3D Olho     ,
					 Vector3D Direcao  ,
					 int      iNumObjs ,
					 Color    CorFundo ,
                int      iProfund  ) ;


/*==============================================================================
*Nome: Render
*Descrio da Funo: dado um cenrio, devolve uma imagem produzida atravs do
*                     algoritmo de ray tracing. 
*
*Parametro recebido
*   pCenario: o cenario. 
*
*Valor Retornado
*   a imagem.
*=============================================================================*/

Image* Render( Scene* pCenario ) ;


/*==============================================================================
*Nome: AcharCompDif
*Descrio da Funo: devolve a componente de reflexo difusa de um ponto da 
*                     imagem. 
*
*Parametros recebidos
*   dLusPosNorm: Normal . LusPos;
*   Difusa:      componente difusa 
*   LuzCor:      cor da luz.
*
*Valor Retornado
*   a cor de reflexo difusa do ponto da imagem.
*=============================================================================*/

Color AcharCompDif( double dLuzPosNorm ,
                    Color  Difusa      ,
                    Color  LuzCor       ) ;


/*==============================================================================
*Nome: AcharCompEsp
*Descrio da Funo: devolve a componente de reflexo especular de um ponto da 
*                     imagem. 
*
*Parametros recebidos
*   Normal:      a normal do ponto da posio;
*   OlhoPos:     o vetor que indica a direo da posio com o olho;
*   LuzPos:      vetor direo da luz com a posio do objeto;
*   dLusPosNorm: Normal . LusPos;
*   dNCoss:      potncia do cosseno;
*   LuzCor:      cor da luz;
*   Especular:   componente especular.
*
*Valor Retornado
*   a cor de reflexo especular do ponto da imagem.
*=============================================================================*/

Color AcharCompEsp( Vector3D Normal      ,
                    Vector3D OlhoPos     ,
                    Vector3D LuzPos      , 
                    double   dLuzPosNorm ,
                    double   dNCoss      , 
                    Color    LuzCor      , 
                    Color    Especular    ) ;


/*==============================================================================
*Nome: iInterceptar
*Descrio da Funo: funo que calcula o ndice do objeto mais prximo
*                     interceptado pelo raio interceptando todos os objetos.
*
*Parametros recebidos
*   pCenario:  o cenario;
*   Olho:      o olho da cmera;
*   Direcao:   a direo do raio;
*   iNumObjs:  nmero de objetos;
*   idxObjExc: objeto excludo do teste de interseptao de raio.
*
*Parmetro retornado:
*   dMinDist: a distncia do menor objeto.
*
*Valor Retornado
*   o ndice do objeto mais prximo interceptado pelo raio.
*=============================================================================*/

int iInterceptar( Scene*   pCenario  ,
                  Vector3D Olho      ,
					   Vector3D Direcao   ,
					   int      iNumObjs  , 
                  int      idxObjExc ,
                  double*  dMinDist   ) ;

#endif


/* render.h */