/** MODIFICADO POR JULIANA **/

#ifndef SCENE_H
#define SCENE_H

#include "object.h"
#include "light.h"
#include "camera.h"

struct Scene_impl;
typedef struct Scene_impl Scene;

/* sceRead - Le uma cena de um arquivo no formato RT2 cujo nome eh dado em
   'filename'. */
Scene * sceRead( const char *filename );

/* sceDestroy - Libera a memoria associada `a cena 'scene' dada. */
void sceDestroy( Scene *scene );

/* sceGetObjectCount - Retorna o numero de objetos na cena 'scene'. */
int sceGetObjectCount( Scene *scene );

/* sceGetObject - Retorna o objeto de indice 'idx' da lista de objetos
   da cena. */
Object * sceGetObject( Scene *scene, int idx );

/* sceGetLightCount - Retorna o numero de fontes de luz na cena 'scene'. */
int sceGetLightCount( Scene *scene );

/* sceGetLight - Retorna a luz de indice 'idx' da lista de luzes
   da cena. */
Light sceGetLight( Scene *scene, int idx );

/* sceGetMaterialCount - Retorna o numero de materiais da cena. */
int sceGetMaterialCount( Scene *scene );

/* sceGetMaterial - Retorna o material de indice 'idx' da cena. */
Material sceGetMaterial( Scene *scene, int idx );


/*==============================================================================
*Nome: PegarCamera
*Descrio da Funo: funo que devolve a camra de um dado cenario.
*
*Parametro recebido
*   pCenario: o cenario
*
*Valor Retornado
*   a camera ou NULL, caso a camera nao tenha sido criada.
*=============================================================================*/

Camera* PegarCamera( Scene* pCenario ) ;


/*==============================================================================
*Nome: PegarCorFundo
*Descrio da Funo: funo que devolve a cor de fundo de um dado cenario.
*
*Parametro recebido
*   pCenario: o cenario
*
*Valor Retornado
*   a cor de fundo.
*=============================================================================*/

Color PegarCorFundo( Scene* pCenario ) ;


#endif
